/**
 * @file motor.cpp
 * @author Keith Holman
 * @date Created: Tuesday, October 14, 2008
 *
 * @brief Defines the atomic model for the motor
 */

#include "motor.h"
#include "message.h"    // class ExternalMessage, InternalMessage
#include "mainsimu.h"   // MainSimulator::Instance().getParameter( ... )
#include <cmath>

Motor::Motor( const std::string& name )
: Atomic( name ),
  _in( addInputPort( "in" ) ),
	_out( addOutputPort( "out" ) ),
  _expelled( 0.0 ),
  _target( 0.0 ),
  _total( 100.0 ),
  _success( true )
{
  _expelled = atof( MainSimulator::Instance().getParameter( description(), "expelled" ).c_str() );
  _total = atof( MainSimulator::Instance().getParameter( description(), "length" ).c_str() );
}

Model &Motor::initFunction()
{
	this->passivate();
	return *this;
}

Model &Motor::externalFunction( const ExternalMessage &msg )
{
	if( msg.port() == _in && state() == passive )
	{
    Value len = msg.value();
    Value delta = 0;
    _success = len > 0 && len < _total;
    if ( _success )
    {
      delta = fabs( len - _expelled );
      _target = len;
    }
    else
    {
      if ( len < 0.0 )
      {
        delta = _expelled;
        _target = 0;
      }
      else
      {
        delta = fabs( _total - _expelled );
        _target = _total;
      }
    }
    unsigned int min = static_cast< unsigned int >( delta / 60 );
    unsigned int sec = static_cast< unsigned int >( ceil( delta ) ) % 60;
    Time t( 0, min, sec, 0 );
    holdIn( active, t );
  }
	return *this;
}

Model &Motor::internalFunction( const InternalMessage & )
{
  _expelled = _target;
	passivate();
	return *this;
}

Model &Motor::outputFunction( const InternalMessage &msg )
{
	sendOutput( msg.time(), _out, _success );
	return *this ;
}

std::string Motor::className() const
{
	return "Motor";
}
