#ifndef POINT_H
#define POINT_H
/**
 * @file point.h
 * @author Keith Holman
 * @date Created: Tuesday, October 14, 2008
 *
 * @brief Declares the Point object type
 */

#include "value.h"

/**
 * @author Keith Holman
 * @date Created: Tuesday, October 14, 2008
 *
 * @brief Object representing a point within the system
 */
class Point
{
public:
  /** @brief Internal type used for the coordinate dimension */
  typedef Value coord_t;

  /**
   * @brief Default constructor
   */
  Point();

  /**
   * @brief Constructor
   * @param x   x-coordinate
   * @param y   y-coordinate
   */
  Point( coord_t x, coord_t y );

  /**
   * @brief Constructor
   * @param v   conversion from CD++ value
   */
  Point( const Value &v );

  /**
   * @brief Returns the x-coordinate
   * @return the x-coordinate value
   */
  coord_t x() const { return _x; }
  /**
   * @brief Sets the x-coordinate
   * @param x   x-coordinate value to set
   */
  void x( coord_t x ) { _x = x; }
  /**
   * @brief Returns the y-coordinate
   * @return the y-coordinate value
   */
  coord_t y() const { return _y; }
  /**
   * @brief Sets the y-coordinate
   * @param y   y-coordinate value to set
   */
  void y( coord_t y ) { _y = y; }

  /**
   * @brief Equality operator
   * @param p   object to compare
   * @retval  true  objects are equal
   * @retval  false objects are not equal
   */
  bool operator==( const Point& p ) const;

  /**
   * @brief Inequality operator
   * @param p   object to compare
   * @retval  true  objects are not equal
   * @retval  false objects are equal
   */
  bool operator!=( const Point& p ) const
  {
    return !operator==( p );
  }

  /**
   * @brief Addition operator
   * @param p   point to add
   * @return an object with the value of @p added to @c this object
   */
  Point operator+ ( const Point& p ) const
  {
    return Point( _x + p._x, _y + p._y );
  }
  /**
   * @brief Subtraction operator
   * @param p   point to subtract
   * @return an object with the value of @p subtracted from @c this object
   */
  Point operator-( const Point& p ) const
  {
    return Point( _x - p._x, _y - p._y );
  }

  /**
   * @brief Addition assignment operator
   * @param p   point to add
   * @return a reference to @c this object
   */
  Point& operator+=( const Point& p );
  /**
   * @brief Subtraction assignment operator
   * @param p   point to subtract
   * @return a reference to @c this object
   */
  Point& operator-=( const Point& p );

  /**
   * @brief Converts a point to the structure that can be sent across a
   * CD++ port
   */
  Value toValue() const;

private:
  /** @brief x-coordinate */
  coord_t _x;
  /** @brief y-coordinate */
  coord_t _y;
};

#endif // ifndef POINT_H
