#ifndef SERIAL_H
#define SERIAL_H
/**
 * @file serial.h
 * @author Keith Holman
 * @date Created: Saturday, October 18, 2008
 *
 * @brief Defines an object for handling serial input on a port
 */

#include <vector>

/**
 * @brief Object for simplifing the serial input of data on a port on
 *        an atomic model
 */
template< typename T, unsigned int N = 2u >
class Serial
{
public:
  /**
   * @brief Constructor
   * @param flag  flag to indicate restarting of new data
   * @param n     number of items that can be recieved serially on the port
   */
  Serial();

  /**
   * @brief Clears the buffer
   */
  void clear() { _next = 0u; }

  /**
   * @brief Whether the object is full of fresh data
   * @retval true   the object is full of data
   * @retval false  the object still has space
   */
  bool full() const
  {
    return ( _next >= N );
  }

  /**
   * @brief Retrieves the internally stored data
   * @return Internally stored data at index @c i
   */
  T get( unsigned int i ) const;

  /**
   * @brief Returns the number of items that can be stored in this
   *        serial input buffer
   * @return  The number of items that can be stored in this serial input
   *          buffer
   */
  unsigned int length() const
  {
    return N;
  }

  /**
   * @brief Called when new data is received on the port
   */
  bool put( T data );


private:
  /**
   * @brief Actual storage object
   */
  T _data[ N ];

  /**
   * @brief Next position to place data in the vector
   */
  unsigned int _next;
};

template< typename T, unsigned int N >
Serial< T, N >::Serial()
  : _next( 0u )
{
}

template< typename T, unsigned int N >
T Serial< T, N >::get( unsigned int i ) const
{
  T result;
  if ( i < N )
  {
    result = _data[ i ];
  }
  return result;
}

template< typename T, unsigned int N >
bool Serial< T, N >::put( T data )
{
  if ( _next < N )
  {
    _data[ _next ] = data;
    ++_next;
  }
  return full();
}

#endif SERIAL_H
