/**
 * @file translator.cpp
 * @author Keith Holman
 * @author Jeremy Kuzub
 * @date Created: Tuesday, October 14, 2008
 *
 * @brief Defines the atomic model for the translator
 */

#include "translator.h"

#include "compare.h"    // for is_equal
#include "message.h"    // class ExternalMessage, InternalMessage
#include "mainsimu.h"   // MainSimulator::Instance().getParameter( ... )
#include <math.h>
#include <cstdlib>
#include <sstream>
#include <iostream>

Translator::Translator( const std::string& name )
: Atomic( name ),
  _in( addInputPort( "in" ) ),
  _done( addInputPort( "done" ) ),
	_out( addOutputPort( "out" ) ),
	_motorOut1( addOutputPort( "motorOut1" ) ),
	_motorOut2( addOutputPort( "motorOut2") ),
  _complete( true ),
  _success( true ),
  _doneCount( 0u )
{
  for ( unsigned int i = 0;
        i < NUMBER_OF_MOTORS; ++i )
  {
    std::ostringstream oss1,oss2;
    _targetLengths[ i ] = 0u;

    oss1 << "anchor" << ( i + 1 ) << ".x";
    oss2 << "anchor" << ( i + 1 ) << ".y";
    _anchorPoints[i].x( atof( MainSimulator::Instance().getParameter( description(), oss1.str() ).c_str() ) );
    _anchorPoints[i].y( atof( MainSimulator::Instance().getParameter( description(), oss2.str() ).c_str() ) );
  }
}

Model &Translator::initFunction()
{
	this->passivate();
	return *this;
}

Model &Translator::externalFunction( const ExternalMessage &msg )
{
	if( msg.port() == _in )
	{
	  _inputData.put( msg.value() );
	  if ( _inputData.full() )
	  {
	    const double PI = 4.0*atan(1.0);
	    // MUST BE CONVERTED TO SUPPORT 3+ OR MORE MOTORS
	    for ( unsigned int i = 0; i < NUMBER_OF_MOTORS; ++i )
	    {
        // Put into active state
        Point::coord_t c[NUMBER_OF_MOTORS];
        // x
        c[0] = _inputData.get( 0 ) - _anchorPoints[i].x();
        // y
        c[1] = _inputData.get( 1 ) - _anchorPoints[i].y();
        double angle = atan2( c[1], c[0] );
        double cos_angle = cos( angle );
        if ( is_equal( cos_angle, 0.0 ) )
          _targetLengths[i] = c[1];   // hanging straight down: set to y
        else
          _targetLengths[i] = c[0] / cos( angle );
	    }
      // clear input buffer
	    _inputData.clear();
      _complete = false;
      _doneCount = 0u;
      holdIn( active, Time( 0, 0, 0, 0 ) );
	  }
	  else
	  {
	    passivate();
	  }
	}
  else if ( msg.port() == _done )
  {
    ++_doneCount;
    if ( _success )
      _success = static_cast< bool >( msg.value() );
    _complete = (_doneCount >= NUMBER_OF_MOTORS);
    if ( _complete )
      holdIn( active, Time( 0, 0, 0, 0 ) );
    else
      passivate();
  }
	return *this;
}

Model &Translator::internalFunction( const InternalMessage & )
{
	passivate();
	return *this;
}

Model &Translator::outputFunction( const InternalMessage &msg )
{
  if ( _complete )
  {
    // Completed translation
	  sendOutput( msg.time(), _out, _success );
  }
  else
  {
    std::cout << _targetLengths[0] << "," << _targetLengths[1] << std::endl;
    sendOutput( msg.time(), _motorOut1, _targetLengths[0] );
    sendOutput( msg.time(), _motorOut2, _targetLengths[1] );
  }
	return *this;
}

std::string Translator::className() const
{
	return "Translator";
}
